/**
 * <toc-observer> highlights the current heading in your
 * table of contents
 *
 * Copyright © 2022 Tony Spegel
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/**
 * @license
 * Copyright 2019 Google LLC
 * SPDX-License-Identifier: BSD-3-Clause
 */
import { LitElement, html } from 'lit';
import { customElement, property, queryAssignedElements, } from 'lit/decorators.js';
/**
 * Observes your TOC elements.
 *
 * @slot - This element has a slot
 */
let TocObserver = class TocObserver extends LitElement {
    constructor() {
        super(...arguments);
        this.rootElement = 'null';
        this.threshold = [0, 1];
        this.tocLinkSelector = '.toc-link';
        this.tocActiveClass = 'toc-active';
        this.observerItemSelector = 'h2[id], h3[id]';
        this.observer = new IntersectionObserver((entries) => {
            entries.forEach((entry) => {
                var _a, _b;
                // Get the id of the intersecting item
                const { id } = entry.target;
                // If any item intersects with our root add / remove a CSS class
                if (entry.intersectionRatio > 0) {
                    (_a = this.selectTocLink(id)) === null || _a === void 0 ? void 0 : _a.classList.add(this.tocActiveClass);
                }
                else {
                    (_b = this.selectTocLink(id)) === null || _b === void 0 ? void 0 : _b.classList.remove(this.tocActiveClass);
                }
            });
        }, 
        // IntersectionObserver options
        {
            root: this.ownerDocument.querySelector(this.rootElement),
            threshold: this.threshold,
        });
    }
    /**
     * Selects a TOC link within the components slot / ul element.
     * Its id param is obtained by reading the oberver item's target-id.
     */
    selectTocLink(id) {
        return this._tocList[0].querySelector(`${this.tocLinkSelector}[href="#${id}"]`);
    }
    get _tocListItems() {
        var _a;
        return ((_a = this._tocList) === null || _a === void 0 ? void 0 : _a.length)
            ? Array.from(this._tocList[0].querySelectorAll(this.tocLinkSelector))
            : null;
    }
    firstUpdated() {
        var _a, _b;
        const observerItems = (_a = this.ownerDocument) === null || _a === void 0 ? void 0 : _a.querySelectorAll(this.observerItemSelector);
        if ((_b = this._tocListItems) === null || _b === void 0 ? void 0 : _b.length) {
            observerItems === null || observerItems === void 0 ? void 0 : observerItems.forEach((item) => this.observer.observe(item));
        }
    }
    disconnectedCallback() {
        this.observer.disconnect();
    }
    render() {
        return html `<slot name="toc"></slot>`;
    }
};
__decorate([
    property({ type: String })
], TocObserver.prototype, "rootElement", void 0);
__decorate([
    property({ type: Array })
], TocObserver.prototype, "threshold", void 0);
__decorate([
    property({ type: String })
], TocObserver.prototype, "tocLinkSelector", void 0);
__decorate([
    property({ type: String })
], TocObserver.prototype, "tocActiveClass", void 0);
__decorate([
    property({ type: String })
], TocObserver.prototype, "observerItemSelector", void 0);
__decorate([
    queryAssignedElements({ slot: 'toc', selector: 'ul' })
], TocObserver.prototype, "_tocList", void 0);
TocObserver = __decorate([
    customElement('toc-observer')
], TocObserver);
export { TocObserver };
//# sourceMappingURL=toc-observer.js.map